"use strict";
/**
 * @description
 * HTTP code snippet generator for native Go.
 *
 * @author
 * @montanaflynn
 *
 * for any questions or issues regarding the generated code snippet, please open an issue mentioning the author.
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
exports.native = void 0;
var code_builder_1 = require("../../../helpers/code-builder");
exports.native = {
    info: {
        key: 'native',
        title: 'NewRequest',
        link: 'http://golang.org/pkg/net/http/#NewRequest',
        description: 'Golang HTTP client request'
    },
    convert: function (_a, options) {
        var postData = _a.postData, method = _a.method, allHeaders = _a.allHeaders, fullUrl = _a.fullUrl;
        var _b = new code_builder_1.CodeBuilder({ indent: '\t' }), blank = _b.blank, push = _b.push, join = _b.join;
        var opts = __assign({ showBoilerplate: true, checkErrors: false, printBody: true, timeout: -1 }, options);
        var errorPlaceholder = opts.checkErrors ? 'err' : '_';
        var indent = opts.showBoilerplate ? 1 : 0;
        var errorCheck = function () {
            if (opts.checkErrors) {
                push('if err != nil {', indent);
                push('panic(err)', indent + 1);
                push('}', indent);
            }
        };
        // Create boilerplate
        if (opts.showBoilerplate) {
            push('package main');
            blank();
            push('import (');
            push('"fmt"', indent);
            if (opts.timeout > 0) {
                push('"time"', indent);
            }
            if (postData.text) {
                push('"strings"', indent);
            }
            push('"net/http"', indent);
            if (opts.printBody) {
                push('"io/ioutil"', indent);
            }
            push(')');
            blank();
            push('func main() {');
            blank();
        }
        // Create client
        var client;
        if (opts.timeout > 0) {
            client = 'client';
            push('client := http.Client{', indent);
            push("Timeout: time.Duration(".concat(opts.timeout, " * time.Second),"), indent + 1);
            push('}', indent);
            blank();
        }
        else {
            client = 'http.DefaultClient';
        }
        push("url := \"".concat(fullUrl, "\""), indent);
        blank();
        // If we have body content or not create the var and reader or nil
        if (postData.text) {
            push("payload := strings.NewReader(".concat(JSON.stringify(postData.text), ")"), indent);
            blank();
            push("req, ".concat(errorPlaceholder, " := http.NewRequest(\"").concat(method, "\", url, payload)"), indent);
            blank();
        }
        else {
            push("req, ".concat(errorPlaceholder, " := http.NewRequest(\"").concat(method, "\", url, nil)"), indent);
            blank();
        }
        errorCheck();
        // Add headers
        if (Object.keys(allHeaders).length) {
            Object.keys(allHeaders).forEach(function (key) {
                push("req.Header.Add(\"".concat(key, "\", \"").concat(allHeaders[key], "\")"), indent);
            });
            blank();
        }
        // Make request
        push("res, ".concat(errorPlaceholder, " := ").concat(client, ".Do(req)"), indent);
        errorCheck();
        // Get Body
        if (opts.printBody) {
            blank();
            push('defer res.Body.Close()', indent);
            push("body, ".concat(errorPlaceholder, " := ioutil.ReadAll(res.Body)"), indent);
            errorCheck();
        }
        // Print it
        blank();
        push('fmt.Println(res)', indent);
        if (opts.printBody) {
            push('fmt.Println(string(body))', indent);
        }
        // End main block
        if (opts.showBoilerplate) {
            blank();
            push('}');
        }
        return join();
    }
};
