"use strict";
/**
 * @description
 * HTTP code snippet generator for native Node.js.
 *
 * @author
 * @AhmadNassri
 *
 * for any questions or issues regarding the generated code snippet, please open an issue mentioning the author.
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.native = void 0;
var stringify_object_1 = __importDefault(require("stringify-object"));
var code_builder_1 = require("../../../helpers/code-builder");
exports.native = {
    info: {
        key: 'native',
        title: 'HTTP',
        link: 'http://nodejs.org/api/http.html#http_http_request_options_callback',
        description: 'Node.js native HTTP interface'
    },
    convert: function (_a, options) {
        var _b;
        var uriObj = _a.uriObj, method = _a.method, allHeaders = _a.allHeaders, postData = _a.postData;
        var opts = __assign({ indent: '  ' }, options);
        var _c = new code_builder_1.CodeBuilder({ indent: opts.indent }), blank = _c.blank, join = _c.join, push = _c.push, unshift = _c.unshift;
        var reqOpts = {
            method: method,
            hostname: uriObj.hostname,
            port: uriObj.port,
            path: uriObj.path,
            headers: allHeaders
        };
        push("const http = require('".concat((_b = uriObj.protocol) === null || _b === void 0 ? void 0 : _b.replace(':', ''), "');"));
        blank();
        push("const options = ".concat((0, stringify_object_1["default"])(reqOpts, { indent: opts.indent }), ";"));
        blank();
        push('const req = http.request(options, function (res) {');
        push('const chunks = [];', 1);
        blank();
        push("res.on('data', function (chunk) {", 1);
        push('chunks.push(chunk);', 2);
        push('});', 1);
        blank();
        push("res.on('end', function () {", 1);
        push('const body = Buffer.concat(chunks);', 2);
        push('console.log(body.toString());', 2);
        push('});', 1);
        push('});');
        blank();
        switch (postData.mimeType) {
            case 'application/x-www-form-urlencoded':
                if (postData.paramsObj) {
                    unshift("const qs = require('querystring');");
                    push("req.write(qs.stringify(".concat((0, stringify_object_1["default"])(postData.paramsObj, {
                        indent: '  ',
                        inlineCharacterLimit: 80
                    }), "));"));
                }
                break;
            case 'application/json':
                if (postData.jsonObj) {
                    push("req.write(JSON.stringify(".concat((0, stringify_object_1["default"])(postData.jsonObj, {
                        indent: '  ',
                        inlineCharacterLimit: 80
                    }), "));"));
                }
                break;
            default:
                if (postData.text) {
                    push("req.write(".concat((0, stringify_object_1["default"])(postData.text, { indent: opts.indent }), ");"));
                }
        }
        push('req.end();');
        return join();
    }
};
