"use strict";
/**
 * @description
 * HTTP code snippet generator for PHP using curl-ext.
 *
 * @author
 * @AhmadNassri
 *
 * for any questions or issues regarding the generated code snippet, please open an issue mentioning the author.
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
exports.curl = void 0;
var code_builder_1 = require("../../../helpers/code-builder");
exports.curl = {
    info: {
        key: 'curl',
        title: 'cURL',
        link: 'http://php.net/manual/en/book.curl.php',
        description: 'PHP with ext-curl'
    },
    convert: function (_a, options) {
        var uriObj = _a.uriObj, postData = _a.postData, fullUrl = _a.fullUrl, method = _a.method, httpVersion = _a.httpVersion, cookies = _a.cookies, headersObj = _a.headersObj;
        var opts = __assign({ closingTag: false, indent: '  ', maxRedirects: 10, namedErrors: false, noTags: false, shortTags: false, timeout: 30 }, options);
        var _b = new code_builder_1.CodeBuilder({ indent: opts.indent }), push = _b.push, blank = _b.blank, join = _b.join;
        if (!opts.noTags) {
            push(opts.shortTags ? '<?' : '<?php');
            blank();
        }
        push('$curl = curl_init();');
        blank();
        var curlOptions = [
            {
                escape: true,
                name: 'CURLOPT_PORT',
                value: uriObj.port
            },
            {
                escape: true,
                name: 'CURLOPT_URL',
                value: fullUrl
            },
            {
                escape: false,
                name: 'CURLOPT_RETURNTRANSFER',
                value: 'true'
            },
            {
                escape: true,
                name: 'CURLOPT_ENCODING',
                value: ''
            },
            {
                escape: false,
                name: 'CURLOPT_MAXREDIRS',
                value: opts.maxRedirects
            },
            {
                escape: false,
                name: 'CURLOPT_TIMEOUT',
                value: opts.timeout
            },
            {
                escape: false,
                name: 'CURLOPT_HTTP_VERSION',
                value: httpVersion === 'HTTP/1.0' ? 'CURL_HTTP_VERSION_1_0' : 'CURL_HTTP_VERSION_1_1'
            },
            {
                escape: true,
                name: 'CURLOPT_CUSTOMREQUEST',
                value: method
            },
            {
                escape: true,
                name: 'CURLOPT_POSTFIELDS',
                value: postData ? postData.text : undefined
            },
        ];
        push('curl_setopt_array($curl, [');
        var curlopts = new code_builder_1.CodeBuilder({ indent: opts.indent, join: "\n".concat(opts.indent) });
        curlOptions.forEach(function (_a) {
            var value = _a.value, name = _a.name, escape = _a.escape;
            if (value !== null && value !== undefined) {
                curlopts.push("".concat(name, " => ").concat(escape ? JSON.stringify(value) : value, ","));
            }
        });
        // construct cookies
        var curlCookies = cookies.map(function (cookie) { return "".concat(encodeURIComponent(cookie.name), "=").concat(encodeURIComponent(cookie.value)); });
        if (curlCookies.length) {
            curlopts.push("CURLOPT_COOKIE => \"".concat(curlCookies.join('; '), "\","));
        }
        // construct cookies
        var headers = Object.keys(headersObj)
            .sort()
            .map(function (key) { return "\"".concat(key, ": ").concat(headersObj[key], "\""); });
        if (headers.length) {
            curlopts.push('CURLOPT_HTTPHEADER => [');
            curlopts.push(headers.join(",\n".concat(opts.indent).concat(opts.indent)), 1);
            curlopts.push('],');
        }
        push(curlopts.join(), 1);
        push(']);');
        blank();
        push('$response = curl_exec($curl);');
        push('$err = curl_error($curl);');
        blank();
        push('curl_close($curl);');
        blank();
        push('if ($err) {');
        if (opts.namedErrors) {
            push('echo array_flip(get_defined_constants(true)["curl"])[$err];', 1);
        }
        else {
            push('echo "cURL Error #:" . $err;', 1);
        }
        push('} else {');
        push('echo $response;', 1);
        push('}');
        if (!opts.noTags && opts.closingTag) {
            blank();
            push('?>');
        }
        return join();
    }
};
