"use strict";
/**
 * @description
 * HTTP code snippet generator for PHP using curl-ext.
 *
 * @author
 * @AhmadNassri
 *
 * for any questions or issues regarding the generated code snippet, please open an issue mentioning the author.
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
exports.http1 = void 0;
var code_builder_1 = require("../../../helpers/code-builder");
var helpers_1 = require("../helpers");
exports.http1 = {
    info: {
        key: 'http1',
        title: 'HTTP v1',
        link: 'http://php.net/manual/en/book.http.php',
        description: 'PHP with pecl/http v1'
    },
    convert: function (_a, options) {
        var method = _a.method, url = _a.url, postData = _a.postData, queryObj = _a.queryObj, headersObj = _a.headersObj, cookiesObj = _a.cookiesObj;
        var opts = __assign({ closingTag: false, indent: '  ', noTags: false, shortTags: false }, options);
        var _b = new code_builder_1.CodeBuilder({ indent: opts.indent }), push = _b.push, blank = _b.blank, join = _b.join;
        if (!opts.noTags) {
            push(opts.shortTags ? '<?' : '<?php');
            blank();
        }
        if (!helpers_1.supportedMethods.includes(method.toUpperCase())) {
            push("HttpRequest::methodRegister('".concat(method, "');"));
        }
        push('$request = new HttpRequest();');
        push("$request->setUrl(".concat((0, helpers_1.convertType)(url), ");"));
        if (helpers_1.supportedMethods.includes(method.toUpperCase())) {
            push("$request->setMethod(HTTP_METH_".concat(method.toUpperCase(), ");"));
        }
        else {
            push("$request->setMethod(HttpRequest::HTTP_METH_".concat(method.toUpperCase(), ");"));
        }
        blank();
        if (Object.keys(queryObj).length) {
            push("$request->setQueryData(".concat((0, helpers_1.convertType)(queryObj, opts.indent), ");"));
            blank();
        }
        if (Object.keys(headersObj).length) {
            push("$request->setHeaders(".concat((0, helpers_1.convertType)(headersObj, opts.indent), ");"));
            blank();
        }
        if (Object.keys(cookiesObj).length) {
            push("$request->setCookies(".concat((0, helpers_1.convertType)(cookiesObj, opts.indent), ");"));
            blank();
        }
        switch (postData.mimeType) {
            case 'application/x-www-form-urlencoded':
                push("$request->setContentType(".concat((0, helpers_1.convertType)(postData.mimeType), ");"));
                push("$request->setPostFields(".concat((0, helpers_1.convertType)(postData.paramsObj, opts.indent), ");"));
                blank();
                break;
            default:
                if (postData.text) {
                    push("$request->setBody(".concat((0, helpers_1.convertType)(postData.text), ");"));
                    blank();
                }
        }
        push('try {');
        push('$response = $request->send();', 1);
        blank();
        push('echo $response->getBody();', 1);
        push('} catch (HttpException $ex) {');
        push('echo $ex;', 1);
        push('}');
        if (!opts.noTags && opts.closingTag) {
            blank();
            push('?>');
        }
        return join();
    }
};
