"use strict";
exports.__esModule = true;
exports.literalRepresentation = void 0;
/**
 * Create an string of given length filled with blank spaces
 *
 * @param length Length of the array to return
 * @param str String to pad out with
 */
function buildString(length, str) {
    return str.repeat(length);
}
/**
 * Create a string corresponding to a Dictionary or Array literal representation with pretty option
 * and indentation.
 */
function concatValues(concatType, values, pretty, indentation, indentLevel) {
    var currentIndent = buildString(indentLevel, indentation);
    var closingBraceIndent = buildString(indentLevel - 1, indentation);
    var join = pretty ? ",\n".concat(currentIndent) : ', ';
    var openingBrace = concatType === 'object' ? '{' : '[';
    var closingBrace = concatType === 'object' ? '}' : ']';
    if (pretty) {
        return "".concat(openingBrace, "\n").concat(currentIndent).concat(values.join(join), "\n").concat(closingBraceIndent).concat(closingBrace);
    }
    return openingBrace + values.join(join) + closingBrace;
}
/**
 * Create a valid Python string of a literal value according to its type.
 *
 * @param {*} value Any JavaScript literal
 * @param {Object} opts Target options
 * @return {string}
 */
var literalRepresentation = function (value, opts, indentLevel) {
    indentLevel = indentLevel === undefined ? 1 : indentLevel + 1;
    switch (Object.prototype.toString.call(value)) {
        case '[object Number]':
            return value;
        case '[object Array]': {
            var pretty_1 = false;
            var valuesRepresentation = value.map(function (v) {
                // Switch to prettify if the value is a dictionary with multiple keys
                if (Object.prototype.toString.call(v) === '[object Object]') {
                    pretty_1 = Object.keys(v).length > 1;
                }
                return (0, exports.literalRepresentation)(v, opts, indentLevel);
            });
            return concatValues('array', valuesRepresentation, pretty_1, opts.indent, indentLevel);
        }
        case '[object Object]': {
            var keyValuePairs = [];
            // eslint-disable-next-line guard-for-in, no-restricted-syntax
            for (var key in value) {
                keyValuePairs.push("\"".concat(key, "\": ").concat((0, exports.literalRepresentation)(value[key], opts, indentLevel)));
            }
            return concatValues('object', keyValuePairs, opts.pretty && keyValuePairs.length > 1, opts.indent, indentLevel);
        }
        case '[object Null]':
            return 'None';
        case '[object Boolean]':
            return value ? 'True' : 'False';
        default:
            if (value === null || value === undefined) {
                return '';
            }
            return "\"".concat(value.toString().replace(/"/g, '\\"'), "\"");
    }
};
exports.literalRepresentation = literalRepresentation;
