"use strict";
/**
 * @description
 * HTTP code snippet generator for R using httr
 *
 * @author
 * @gabrielakoreeda
 *
 * for any questions or issues regarding the generated code snippet, please open an issue mentioning the author.
 */
exports.__esModule = true;
exports.httr = void 0;
var code_builder_1 = require("../../../helpers/code-builder");
exports.httr = {
    info: {
        key: 'httr',
        title: 'httr',
        link: 'https://cran.r-project.org/web/packages/httr/vignettes/quickstart.html',
        description: 'httr: Tools for Working with URLs and HTTP'
    },
    convert: function (_a) {
        var url = _a.url, queryObj = _a.queryObj, queryString = _a.queryString, postData = _a.postData, allHeaders = _a.allHeaders, method = _a.method;
        // Start snippet
        var _b = new code_builder_1.CodeBuilder(), push = _b.push, blank = _b.blank, join = _b.join;
        // Import httr
        push('library(httr)');
        blank();
        // Set URL
        push("url <- \"".concat(url, "\""));
        blank();
        // Construct query string
        var qs = queryObj;
        delete queryObj.key;
        var queryCount = Object.keys(qs).length;
        if (queryString.length === 1) {
            push("queryString <- list(".concat(Object.keys(qs), " = \"").concat(Object.values(qs).toString(), "\")"));
            blank();
        }
        else if (queryString.length > 1) {
            push('queryString <- list(');
            Object.keys(qs).forEach(function (query, i) {
                if (i !== queryCount - 1) {
                    push("  ".concat(query, " = \"").concat(qs[query].toString(), "\","));
                }
                else {
                    push("  ".concat(query, " = \"").concat(qs[query].toString(), "\""));
                }
            });
            push(')');
            blank();
        }
        // Construct payload
        var payload = JSON.stringify(postData.text);
        if (payload) {
            push("payload <- ".concat(payload));
            blank();
        }
        // Define encode
        if (postData.text || postData.jsonObj || postData.params) {
            switch (postData.mimeType) {
                case 'application/x-www-form-urlencoded':
                    push('encode <- "form"');
                    blank();
                    break;
                case 'application/json':
                    push('encode <- "json"');
                    blank();
                    break;
                case 'multipart/form-data':
                    push('encode <- "multipart"');
                    blank();
                    break;
                default:
                    push('encode <- "raw"');
                    blank();
                    break;
            }
        }
        // Construct headers
        var headers = [];
        var cookies;
        var accept;
        Object.keys(allHeaders).forEach(function (header) {
            if (header.toLowerCase() === 'accept') {
                accept = ", accept(\"".concat(allHeaders[header], "\")");
            }
            else if (header.toLowerCase() === 'cookie') {
                cookies = ", set_cookies(`".concat(String(allHeaders[header])
                    .replace(/;/g, '", `')
                    .replace(/` /g, '`')
                    .replace(/[=]/g, '` = "'), "\")");
            }
            else if (header.toLowerCase() !== 'content-type') {
                headers.push("'".concat(header, "' = '").concat(allHeaders[header], "'"));
            }
        });
        // Construct request
        var request = "response <- VERB(\"".concat(method, "\", url");
        if (payload) {
            request += ', body = payload';
        }
        if (headers.length) {
            request += ", add_headers(".concat(headers.join(', '), ")");
        }
        if (queryString.length) {
            request += ', query = queryString';
        }
        request += ", content_type(\"".concat(postData.mimeType, "\")");
        if (typeof accept !== 'undefined') {
            request += accept;
        }
        if (typeof cookies !== 'undefined') {
            request += cookies;
        }
        if (postData.text || postData.jsonObj || postData.params) {
            request += ', encode = encode';
        }
        request += ')';
        push(request);
        blank();
        // Print response
        push('content(response, "text")');
        return join();
    }
};
